@@ -1,5 +1,7 @@
 package org.af.gMCP.config;
 
+import javax.swing.UIManager;
+
 
 public class JavaConfig extends SpecificConfig {
 
@@ -12,7 +14,7 @@ public class JavaConfig extends SpecificConfig {
     }
     
     public String getLooknFeel() {
-        return getProperty("looknfeel", "com.jgoodies.looks.plastic.Plastic3DLookAndFeel");
+        return getProperty("looknfeel", UIManager.getCrossPlatformLookAndFeelClassName()); // "com.jgoodies.looks.plastic.Plastic3DLookAndFeel");
     }
 
     public void setLooknFeel(String looknfeel) {
